//Any clickable item that appears on the game screen. Image determines mouse detection.
class Clickable {
  PImage clk_image;
  int clk_height;
  int clk_width;
  int clk_x;
  int clk_y;
  
  Clickable(String filename){
    String file = "/images/" + filename;
    clk_image = loadImage(file);
    clk_height = clk_image.height;
    clk_width = clk_image.width;
  }
  
  //Checks for click
  boolean mouseOver(){
    return (mouseX > this.getX() && mouseX < this.getX()+this.getWidth() &&
       mouseY > this.getY() && mouseY < this.getY()+this.getHeight());
  }
  
  void onClick(){
    clearChoices();
  }
  
  PImage getImage() {
    return clk_image;
  }
  
  int getHeight() {
    return clk_height;
  }
  
  int getWidth() {
    return clk_width;
  }
  
  int getX() {
    return clk_x;
  }
  
  int getY() {
    return clk_y;
  }
    
}

//Objects in the game world, including people subclass)
class Object extends Clickable {
  String desc;
  String measure;
  String weigh;
  String mathhint;

  
  Object(String filename) {
    super(filename);
    desc = "";
    measure = "";
    weigh = "";
    mathhint = "";
  }
  
  void onClick(){ //Check the current tool to see what information to get
   clearChoices();
   
   if (this.mathhint != "") {
     math_hint = this.mathhint;
   } else {
     math_hint = cur_room.mathhint;
   }
    
   cur_object = this;
   switch(cur_tool) {
    case "lens":
      this.getDesc();
      break;
    case "measure":
      this.getMeasure();
      break;
    case "weigh":
      this.getWeigh();
      break;
    }
  }
  
  void getDesc() {
    setStatus(desc);
  }
  
  void getMeasure() {
    setStatus(measure);
  }
  
  void getWeigh() {
    setStatus(weigh);
  }
  
}

//Doors are Clickables that go to rooms when clicked
class Door extends Clickable {
  Room destination;
  String type;
  
  Door(String filename, Room dest, String sound_type){
    super(filename);
    destination = dest;
    type = sound_type;
  }
  
  void onClick(){
    goToRoom(destination);
    if (type == "footstep"){
      stepsound.rewind();
      stepsound.play(1000);
    } else if (type == "door") {
      doorsound.rewind();
      doorsound.play();
    }
    
  }
}

void mouseClicked(){
    
    boolean done = false; //The "done" system prevents "click-through" when an object in the next room is at the same location
    //Find the object clicked on in the room
    for (int i = 0; i < cur_room.num_obj; i++) {
     if (!done && cur_room.obj[i].mouseOver()){
       cur_room.obj[i].onClick();
       done = true;
     }
    }
    
    //Or find the the choice clicked in the dialogue area
    for (int i = 0; i < 4; i++) {
     if (!done && choiceArray[i].mouseOver()){
       choiceArray[i].onClick();
       done = true;
     }
    }
    
    //Or the GUI
    for (int i = 0; i < GUI.num_obj; i++) {
     if (cur_room.showGUI && !done && GUI.obj[i].mouseOver()){
       GUI.obj[i].onClick();
       done = true;
     }
    }

}


void drawChoices(){
  
  for (int i = 0; i < 4; i++){
      if (choiceArray[i].choiceText != "") {
        image(ChoiceIcon, choiceX[i], 560);
        fill(#8F334D);
        text(choiceArray[i].choiceText, choiceX[i]+25, 570);
      }
  }
}

void keyPressed() {
  
  //Number hotkeys for tools: 1 = lens, 2 = talk, 3 = measure, 4 = weigh
  switch(key) {
    case '1':
      cur_tool = "lens";
      cursor(GlassCursor, 0, 0);
      break;
    case '2':
      cur_tool = "talk";
      cursor(TalkCursor, 0, 0);
      break;
    case '3':
      cur_tool = "measure";
      cursor(MeasureCursor, 0, 0);
      break;
    case '4':
      cur_tool = "weigh";
      cursor(WeighCursor, 0, 0);
      break;
    case 'm':
      if (music.isPlaying()){
        music.pause();
      } else {
        music.loop();
      }
      break;
    //case 'p':
    //  goToRoom(paintings);
    //  break;
    //case 'l':
    //  goToRoom(lobby);
    //  break;
    //case 'm':
    //  goToRoom(modern);
    //  break;
    //case 'r':
    //  goToRoom(relics);
    //  break;
    //case 'n':
    //  goToRoom(natural_history);
    //  break;
    //case 's':
    //  goToRoom(security);
    //  break;
    //case 'g':
    //  goToRoom(giftshop);
    //  break;
    //case 'c':
    //  goToRoom(closet);
    //  break;
    //case 'e':
    //  goToRoom(cafe);
    //  break;
    //case 'f':
    //  goToRoom(FullFootprint);
    //  break;
    //case 'h':
    //  clearChoices();
    //  setStatus("Ada: \"" + math_hint + "\"");
    //  break;
    //case '/':
    //  goToRoom(intro);
    //  break;
    case 'z':
      goToRoom(Report);
      cp5.show();
      break;
    //case 'i':
    //  getCaseHint();
      
    //  break;
      
    }
  }
